# encoding: utf-8
import es
import gamethread
import effectlib
               
######################
# Class and Methods

'''
This is a dictionary to store the instances of
all the beacons in. The reason we need the instances
is because theyre not global, so we define them
in a dictionary and return them later on.
'''
beacon_users = {}

estools = es.exists("variable", "est_version")
sas_beacon_size = es.ServerVar('sas_beacon_size', 0, "sas : Rayon du balisage")
sas_beacon_r = es.ServerVar('sas_beacon_r', 0, "sas : Couleur Rouge, 0 à 255")
sas_beacon_v = es.ServerVar('sas_beacon_v', 0, "sas : Couleur Vert, 0 à 255")
sas_beacon_b = es.ServerVar('sas_beacon_b', 0, "sas : Couleur Bleu, 0 à 255")
sas_beacon_team = es.ServerVar('sas_beacon_team', 0, "sas : La couleur du balisage et de la couleur de la Team du joueur meme")

class Beacon:
    "Main class to control beacons."
    def __init__(self, userid):
        '''
        This runs whenever the Beacon class is called...
        e.g. a = Beacon() will call this __init__ method.
        It defines all the default options, and allows the
        player to change default options for EasyBeacon...
        Each one of these options are specific to its
        instance. E.G, a = Beacon() will have a different
        set of values to b = Beacon().
        '''
        self.repeat = 0
        self.delay = 0
        self.userid = str(userid)
        self.sound = "sas/sas_beacon.wav"
        self.model = "materials/sprites/lgtning.vmt"
        
        if sas_beacon_team == 1:
            if es.getplayerteam(self.userid) == 2:
                self.color = "255 0 0 255"
            if es.getplayerteam(self.userid) == 3:
                self.color = "0 0 255 255"
        else:
            self.color = "%d %d %d 255" % (int(sas_beacon_r), int(sas_beacon_v), int(sas_beacon_b))
        self.Type = "Normal"
        self.Function = None
       
    #def Pause(self):
    #    '''
    #    This class just cancels the current delay of
    #    the beacon. The repeat and delay values are
    #    not reset, so the next time Resume is hit, it
    #    will carry on from where it left off.
    #    '''
    #    gamethread.cancelDelayed(self.userid + '_beacon')
       
    #def Resume(self, delay=None):
    #    '''
    #    Here we make an optional parameter; the delay.
    #    We specify the default value to None, so that
    #    if no delay is sent in a parameter, then we
    #    pick up the if not delay:, and the delay is
    #    equal to its previous value, which is defined
    #    withing self.delay
    #    '''
    #    if not delay:
    #        delay = self.delay
    #    gamethread.delayedname(delay, self.userid + '_beacon', self.Loop)
       
    def Start(self, delay=1, repeat = "INDEFINITELY"):
        '''
        Okay, this is to start the beacon looping...
        The delay is the delay between loops, and the
        repeat is how many times to repeat. Both values
        by default will loop the beacon every second until
        it is either stopped by a command, or the player dies,
        the round ends or the player disconnects...
        '''
        self.delay = delay
        self.repeat = repeat
        gamethread.delayedname(delay, self.userid + '_beacon', self.Loop)
   
    def Stop(self, delete = False):
        '''
        Here we force the loop to stop, and the delete
        parameter, if true, will also delete the beacon
        instance to clear up a bit of space. Because the
        beacon stops, not pauses, we reset the repeat and
        delay back to 0. We also cancel the current delay
        on the beacon.
        '''
        self.repeat = 0
        self.delay = 0
        gamethread.cancelDelayed(self.userid + '_beacon')
        print delete
        #if delete:
        Delete(self.userid)
       
    def Loop(self):
        '''
        This is the actuall looping of the beacon. Beacause
        there are 2 types, we define the easy beacon, and also
        put a couple of checks for the return function if its
        a normal beacon loop.
        '''
        if not self.repeat == "INDEFINITELY" and self.repeat > 0:
            ''' Okay, we want to ignore if the repeat is set to repeat forever '''
            self.repeat -= 1
        elif not self.repeat == "INDEFINITELY":
            ''' If the repeat is smaller than 0, then stop the loop and return '''
            self.Stop(True)
            return
        if not es.getplayerprop(self.userid, 'CBasePlayer.pl.deadflag'):
            ''' Player is not dead '''
            if self.Type == "Easy":
                '''
                If the type is an easy beacon, retrieve the X,Y,Z of the player
                and create a couple of simple circles, and emit a sound. Both use
                the self.model and self.sound values, so the player can change these.
                '''
                r, g, b, a = str(self.color).split(' ')
                location = es.getplayerlocation(self.userid)
                if estools == 0:
                    effectlib.drawCircle(location, 150, steps=50, model=self.model, seconds=0.2, width=20, endwidth=10, red=r, green=g, blue=b, brightness = a, speed=70)
                else:
                    es.server.queuecmd('est_effect 10 #a 0 "sprites/lgtning.vmt" %s %s %s 1 %d 1 10 50 0 %d %d %d %d 0' % (location[0], location[1], location[2], sas_beacon_size, int(r), int(g), int(b), int(a)))
                if self.sound:
                    es.emitsound('player', self.userid, self.sound, 1.0, 0.7)
            elif self.Type == "Normal":
                ''' Otherwise, if the type of the beacon loop is normal '''
                if callable(self.Function):
                    '''
                    We check if the function is a correct function, if so, we call
                    it and pass the userid as a function.
                    '''
                    self.Function(self.userid)
                else:
                    ''' If the function isnt available, try to es.doblock it '''
                    es.doblock(self.Function)
            '''
            After all these checks have occured, make another delay to run this loop
            again. Call the delay <userid>_beacon, so we can cancel it at later stages.
            The delay is the beacons instances self value.
            '''
            gamethread.delayedname(self.delay, self.userid + '_beacon', self.Loop)
        else:
            '''
            The player is dead, so stop the looping
            '''
            self.Stop(True)
           
    def Delete(self):
        '''
        This is here just incase a player types
        a = beaconlib.EasyBeacon(userid)
        a.Delete(). Its just a wrapper for the
        Stop method.
        '''
        self.Stop(True)

'''
The following methods are here for ease of use. They wrap the
classes methods, and produce easy, script-friendly methods...
Each one of these methods deal with the instance of the beacon
inside the dictionary.
'''

def EasyBeacon(userid):
    '''
    This is to create a simple easy beacon, with no messing.
    It uses default values, which will not use est.
    '''
    userid = str(userid)
    if not beacon_users.has_key(userid):
        '''
        The dictionary does not have an instance of the userid,
        so we create one now. We stor the instance Beacon(userid)
        inside this dictionary, and make the self.Type "Easy".
        '''
        beacon_users[userid] = Beacon(userid)
        beacon_users[userid].Type = "Easy"
    ''' Finally, return the instance of the class. '''
    return beacon_users[userid]
   
#def Create(userid, ReturnFunction):
#    '''
#    This creates a new instance of a beacon, stores it inside
#    the beacon_users dictionary, and adds the .Function to the
#    ReturnFunction parameter. This sets it up so that each loop
#    will run the function the user defines.
#    '''
#    userid = str(userid)
#    if not beacon_users.has_key(userid):
#        beacon_users[userid] = Beacon(userid)
#        beacon_users[userid].Function = ReturnFunction
#    return beacon_users[userid]
   
def Delete(userid):
    '''
    This function checks to see if the beacon_users dictionary
    has the key of the users ID. If so, the cancel the current
    delay, and delete the players instance from the dictionary.
    '''
    userid = str(userid)
    if not beacon_users.has_key(userid):
        es.dbgmsg(0, 'ERROR: Cannot find instance %s inside beacon'%userid)
    else:
        gamethread.cancelDelayed(userid + '_beacon')
        del beacon_users[userid]

#def Find(userid):
#    '''
#    This enables a scriptor to find the same instance at a later
#    stage. Because most variables are not global, whenever they
#    leave the function they created the instance in, that variable
#    will most likely not be global. For this reason, they can obtain
#    the instance back via this method.
#    '''
#    userid = str(userid)
#    if not beacon_users.has_key(userid):
#        es.dbgmsg(0, 'ERROR: Cannot find instance %s inside beacon'%userid)
#    else:
#        return beacon_users[userid]

######################
# Events
# Get this library handle for registering events dynamically:
selfmodule = __import__('beaconlib')

def player_disconnect(ev):
    '''
    When a player disconnects, check for his ID inside the dictionary,
    if there is a value there, call the cancel delay, and delete the
    instance.
    '''
    userid = ev['userid']
    if beacon_users.has_key(userid):
        gamethread.cancelDelayed(userid + '_beacon')
        Delete(userid)
es.addons.registerForEvent(selfmodule, 'player_disconnect', player_disconnect)
       
def player_death(ev):
    ''' Same as player_disconnect '''
    userid = ev['userid']
    if beacon_users.has_key(userid):
        gamethread.cancelDelayed(userid + '_beacon')
        Delete(userid)
es.addons.registerForEvent(selfmodule, 'player_death', player_death)
       
def round_end(ev):
    '''
    As a player may not die during the round, this method checks
    for all alive players at the end of the round, then deletes
    them accordingly.
    '''
    for userid in es.getUseridList():
        if not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
            userid = str(userid)
            if beacon_users.has_key(userid):
                Delete(userid)
es.addons.registerForEvent(selfmodule, 'round_end', round_end)

def server_cvar(event_var):
    if event_var['cvarname'] == 'mp_restartgame' and int(event_var['cvarvalue']) == 1:
        for userid in beacon_users.keys():
            gamethread.cancelDelayed(userid + '_beacon')
            Delete(userid)
es.addons.registerForEvent(selfmodule, 'server_cvar', server_cvar)

def round_start(event_var):
    for userid in beacon_users.keys():
        gamethread.cancelDelayed(userid + '_beacon')
        Delete(userid)
es.addons.registerForEvent(selfmodule, 'round_start', round_start)

